

LIBRARY IEEE;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_1164.all;

ENTITY PWM_Gen IS

GENERIC(

	PWM_min				: integer	:= 0;			-- Min. PWM value in %
	PWM_max				: integer	:= 65			-- Max. PWM value in %
);

PORT(
	i_Clock_150MHz		: IN	std_logic;
	i_n_Reset			: IN	std_logic;						 -- Reset
 	i_pwm_en    		: IN	std_logic;						 -- 0: PWM Disabled - 1: PWM enabled
	i_pwm_pulse_en    : IN	std_logic;                  -- PWM Enable für Anzahl der Pulse funktion aktivieren
	i_pwm_pulse_run   : IN  std_logic;                  -- Trigger für Anzahl der gewünschten Pulse erzeugen
	i_Error        	: IN  std_logic;                  -- Error  
	i_LockPWM        	: IN  std_logic;                  -- Lock PWM without error 
	i_pwm_pulse_count : IN  integer range 0 to 65535;   -- PWM Anzahl der gewünschten Pulse
	i_PWM_Freq_Devider: IN  integer range 0 to 65535;   -- PWM frequency devider (f_pwm = 150MHz / i_PWM_Freq_Devider) -- f_pwm = 20KHz
	i_PWMvalue			: IN	integer range 0 to 8192;	 -- PWM value in 100%/i_PWM_Freq_Devider
	i_PWMman				: IN	std_logic;						 -- 1: Set PWM value manual
	i_PWMmanVal			: IN	integer range 0 to 8192;	 -- PWM value in 100%/i_PWM_Freq_Devider
	o_PWM				   : OUT	std_logic;						 -- PWM signale	
	o_Reset_LockPWM	: OUT	std_logic						 -- Reset lock	
);

END ENTITY PWM_Gen;

ARCHITECTURE behaviour OF PWM_Gen IS

	signal s_PWMCounter		: integer range 0 to 8192;	-- Counter to create the PWM value
	signal s_updatePWMvalue			: std_logic;					-- Set new PWM value
BEGIN

ASSERT (PWM_min >= 0 AND PWM_min < PWM_max)
	REPORT "PWM_min value is wrong! PWM_min = 0..PWM_max"
	SEVERITY error;
ASSERT (PWM_max > PWM_min AND PWM_max <= 100)
	REPORT "PWM_max value is wrong! PWM_min = PWM_min..100"
	SEVERITY error;
	

PWMControl : PROCESS(i_n_Reset, i_Clock_150MHz, i_pwm_en, i_pwm_pulse_run, i_Error, i_LockPWM, i_pwm_pulse_count, i_PWM_Freq_Devider)

	variable v_PWMout						: std_logic;					-- PWM signal
	variable v_intPWMvalue				: integer range 0 to 8192;	-- PWM value in 100%/i_PWM_Freq_Devider (internal value)
	variable	v_updatePWMvalue			: std_logic;					-- Set new PWM value
	variable v_pwm_pulse_count 		: integer range 0 to 65535;
	variable v_pulse_count_nom 		: integer range 0 to 65535;
	variable v_pwm_pulse_run   		: std_logic;
	variable v_Reset_LockPWM_count 	: integer range 0 to 7;
	variable v_Reset_LockPWM		 	: std_logic;
	
BEGIN  
	
	IF i_n_Reset = '0' THEN
		-- Asynchronous reset
		o_PWM					<= '0';
		s_PWMCounter 		<= 0;
		v_updatePWMvalue	:= '0';
		 
	ELSIF rising_edge(i_Clock_150MHz) THEN
		-- Increment the PWM counter
		
	   IF (i_Error = '0') THEN 	
		   
			IF i_pwm_pulse_run = '0' THEN
					v_pulse_count_nom := i_pwm_pulse_count;
					v_pwm_pulse_count	:= 0;
					v_pwm_pulse_run := '0';
		   END IF;		
			
			IF s_PWMCounter < i_PWM_Freq_Devider - 1 THEN
				s_PWMCounter 		<= s_PWMCounter + 1;
			ELSE
				s_PWMCounter 				<= 0;
				v_Reset_LockPWM			:= '1';
				v_Reset_LockPWM_count	:= 7;
			  
				IF i_pwm_pulse_en = '1' AND  v_pwm_pulse_count < v_pulse_count_nom +1 and i_pwm_pulse_run = '1' THEN			
						v_pwm_pulse_count := v_pwm_pulse_count + 1;
						v_pwm_pulse_run := '1';
				END IF;
			END IF;
			
			IF v_Reset_LockPWM = '1' THEN
				IF v_Reset_LockPWM_count > 0 THEN
					v_Reset_LockPWM_count	:= v_Reset_LockPWM_count - 1;
				ELSE
					v_Reset_LockPWM := '0';
				END IF;
			END IF;
			
			
			IF v_updatePWMvalue = '1' THEN
				IF i_PWMman = '0' THEN
					IF i_PWMvalue < PWM_min * (i_PWM_Freq_Devider / 100) THEN
						v_intPWMvalue	:= PWM_min * (i_PWM_Freq_Devider / 100);
					ELSIF i_PWMvalue > PWM_max * (i_PWM_Freq_Devider / 100) THEN
						v_intPWMvalue	:= PWM_max * (i_PWM_Freq_Devider / 100);
					ELSE
						v_intPWMvalue	:= i_PWMvalue;
					END IF;
				ELSE
					v_intPWMvalue	:= i_PWMmanVal;
				END IF;
			END IF;
			
         -- Enables the funtion to generate desired number of PWM Pulses
			IF(i_pwm_pulse_en = '1') THEN  
				IF (v_pwm_pulse_run = '1') THEN
					IF i_pwm_en = '0' AND i_PWMman = '0' THEN
						v_PWMout				:= '0';
						v_updatePWMvalue	:= '1';
					ELSIF v_pwm_pulse_count = v_pulse_count_nom +1 THEN -- Count of PWM Pulses 
						s_PWMCounter		<= 0;
						v_PWMout				:= '0';
						v_updatePWMvalue	:= '1';	
					ELSIF v_intPWMvalue = 0 THEN
						s_PWMCounter		<= 0;
						v_PWMout				:= '0';
						v_updatePWMvalue	:= '1';	
					ELSIF v_intPWMvalue = i_PWM_Freq_Devider THEN 
						s_PWMCounter		<= 0;
						v_PWMout				:= '1';
						v_updatePWMvalue	:= '1';
					ELSIF s_PWMCounter < v_intPWMvalue THEN 
						v_PWMout				:= '1';
						v_updatePWMvalue	:= '0';
					ELSIF s_PWMCounter = v_intPWMvalue THEN
						v_PWMout				:= '0';
						v_updatePWMvalue	:= '1';	   		    				
					ELSE
						v_PWMout				:= '0';
						v_updatePWMvalue	:= '1';
					END IF;
					
					
					o_PWM <= v_PWMout;
				ELSE
					o_PWM <= '0';
				END IF;	
				
			ELSE
			
                o_Reset_LockPWM 	<= v_Reset_LockPWM;
			END IF;
		ELSE
			o_PWM 				<= '0';
			o_Reset_LockPWM	<= '1';
		END IF;
	END IF;
END PROCESS PWMControl;

END ARCHITECTURE behaviour;